// Copyright (c) Jupyter Development Team.
// Copyright (c) Microsoft Corporation.
// Distributed under the terms of the Modified BSD License.

// FIXME
test('temporary test', () => {
  const sum = (a: number, b: number): number => {
    return a + b;
  };
  expect(sum(1, 2)).toBe(3);
});
