// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.

import { test, expect } from '@playwright/test';

test('Default', async ({ page }) => {
  await page.goto('/iframe.html?id=components-combobox--default');

  expect(await page.locator('jp-combobox').screenshot()).toMatchSnapshot(
    'combobox-default.png'
  );
});

test('Disabled', async ({ page }) => {
  await page.goto('/iframe.html?id=components-combobox--with-disabled');

  expect(await page.locator('jp-combobox').screenshot()).toMatchSnapshot(
    'combobox-disabled.png'
  );
});

test('Custom Indicator', async ({ page }) => {
  await page.goto('/iframe.html?id=components-combobox--with-custom-indicator');

  expect(await page.locator('jp-combobox').screenshot()).toMatchSnapshot(
    'combobox-custom-indicator.png'
  );
});

test('Error', async ({ page }) => {
  await page.goto('/iframe.html?id=components-combobox--with-error');

  expect(await page.locator('jp-combobox').screenshot()).toMatchSnapshot(
    'combobox-error.png'
  );
});
