import { create } from '@storybook/theming/create';

export default create({
  base: 'dark',
  brandTitle: 'Jupyter UI Toolkit',
  brandUrl: 'https://github.com/jupyterlab-contrib/jupyter-ui-toolkit'
});
